<?php

namespace Drupal\project_wiki_markdown_content\Plugin;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\project_wiki\Plugin\ProjectWikiContentPluginBase;
use Drupal\project_wiki\ProjectWikiValueObject;
use League\CommonMark\CommonMarkConverter;
use League\CommonMark\Extension\FrontMatter\FrontMatterExtension;

/**
 * The PluginBase for the Markdown services.
 */
abstract class ProjectWikiMarkdownContentPluginBase extends ProjectWikiContentPluginBase {
  use StringTranslationTrait;

  /**
   * Abstract definition for the function providing directory path.
   */
  abstract public function getEntriesDirectoryPath();

  /**
   * {@inheritdoc}
   */
  public function getValueObjects(): array {
    $entries = [];
    $searchDir = $this->getEntriesDirectoryPath();
    $files = $this->readMarkdownDirectories($searchDir);
    foreach ($files as $file) {
      $langcode = explode(DIRECTORY_SEPARATOR, str_replace($searchDir . DIRECTORY_SEPARATOR, '', $file->uri), 2)[0];
      $category = explode(DIRECTORY_SEPARATOR, explode(DIRECTORY_SEPARATOR, str_replace($searchDir . DIRECTORY_SEPARATOR, '', $file->uri), 2)[1], 2)[0];
      if (str_ends_with($category, '.md')) {
        $category = $this->t('No category');
      }
      // Turn markdown into HTML and read header data:
      $entries[] = $this->readMarkdownFile($file, $langcode ?: NULL, $category ?: NULL);
    }
    return $entries;
  }

  /**
   * Returns all markdown files in the given directory and its subdirectories.
   *
   * @param string $searchDir
   *   The directory to search in.
   * @param string $mask
   *   The mask to search by. Here: .md files.
   *
   * @return array
   *   The list of files.
   */
  public function readMarkdownDirectories(string $searchDir, $mask = "/\.md$/") {
    /** @var \Drupal\Core\File\FileSystemInterface $file_system */
    $file_system = \Drupal::service('file_system');
    $files = [];
    $allFiles = [];
    if (is_dir($searchDir)) {
      $allFiles = $file_system->scanDirectory($searchDir, $mask);
    }
    else {
      throw new \Exception('Directory not found: "' . $searchDir . '"');
    }
    foreach ($allFiles as $file) {
      if (file_exists($file->uri)) {
        $files[] = $file;
      }
    }
    return $files;
  }

  /**
   * Reads a markdown file.
   *
   * Reads the given markdown file and turns
   * it into an ProjectWikiEntityContentValueObject.
   */
  public function readMarkdownFile($file, $langcode = NULL, $category = NULL) {
    $fileContents = file_get_contents($file->uri);
    $markdown = $this->parseMarkdown($fileContents);
    $provider = 'project_wiki_markdown_content';
    $id = $file->uri;
    $langcode = $langcode;
    $category = $category;
    if ($markdown['metadata']['title'] == NULL) {
      $title = $file->name;
    }
    else {
      $title = $markdown['metadata']['title'];
    }
    if ($markdown['metadata']['isDeveloperContent'] == NULL) {
      $isDeveloperContent = FALSE;
    }
    else {
      $isDeveloperContent = $markdown['metadata']['isDeveloperContent'];
    }
    $content = $markdown['htmlContent'];
    return new ProjectWikiValueObject($provider, $id, $langcode, $category, $title, $content, $isDeveloperContent);
  }

  /**
   * Helper function to turn markdown string into metadata and html array.
   *
   * @param string $markdown
   *   The markdown string to parse.
   */
  public function parseMarkdown(string $markdown) {
    if (\Drupal::config('project_wiki_markdown_content.settings')->get('allowUnsafeLinks') == 1) {
      $unsafeLinks = TRUE;
    }
    else {
      $unsafeLinks = FALSE;
    }
    $converter = new CommonMarkConverter([
      'html_input' => \Drupal::config('project_wiki_markdown_content.settings')->get('htmlFilter'),
      'allow_unsafe_links' => $unsafeLinks,
    ]);
    $converter->getEnvironment()->addExtension(new FrontMatterExtension());
    /** @var League\CommonMark\Output\RenderedContentInterface $htmlContent */
    $htmlContent = $converter->convert($markdown);
    $isDeveloperContent = $htmlContent->getFrontMatter()['isDeveloperContent'];
    $title = $htmlContent->getFrontMatter()['title'];
    return [
      'metadata' => [
        'title' => $title,
        'isDeveloperContent' => $isDeveloperContent,
      ],
      'htmlContent' => $htmlContent,
    ];
  }

}
